"use strict";

var logger = require('../logger');
var GuiConnector = require('../GuiConnector');

/**
 * This class is responsible for touch interface on the map.
 */
function TouchMap(paramCustomMap) {
  this._customMap = paramCustomMap;
  this.map = paramCustomMap.getGoogleMap();

  logger.info("Turning on custom touch interfaces");
  var self = this;
  var el = this.map.getDiv();

  el.addEventListener('touchstart', function(evt) {
    self.handleStart(evt);
  }, true);
  el.addEventListener("touchend", function(evt) {
    self.handleEnd(evt);
  }, true);
  el.addEventListener("touchcancel", function(evt) {
    self.handleCancel(evt);
  }, true);
  el.addEventListener("touchleave", function(evt) {
    self.handleEnd(evt);
  }, true);
  el.addEventListener("touchmove", function(evt) {
    self.handleMove(evt);
  }, true);

  this.clearTouchData();
  this.rightMenuOn = false;
  this.latLng = new google.maps.LatLng(0.0, 0.0);

  google.maps.event.addListener(this.map, 'mouseover', function(mouseEvent) {
    self.latLng = mouseEvent.latLng;
  });
  google.maps.event.addListener(this.map, 'mousemove', function(mouseEvent) {
    self.latLng = mouseEvent.latLng;
  });
  google.maps.event.addListener(this.map, 'zoom_changed', function() {
    self.getCustomMap().refreshMarkers();
  });

}

TouchMap.prototype.getCustomMap = function() {
  return this._customMap;
};

TouchMap.prototype.clearTouchData = function() {
  this.firstFingerId = null;
  this.firstStartX = null;
  this.firstStartY = null;

  this.firstEndX = null;
  this.firstEndY = null;

  this.secondFingerId = null;
  this.secondStartX = null;
  this.secondStartY = null;
  this.secondEndX = null;
  this.secondEndY = null;

  this.startCoord = null;
  this.startZoom = null;
  this.ongoingTouches = [];

  this.lastMoveDx = 0;
  this.lastMoveDy = 0;

  this.lastStartedFinger = null;
  this.lastStartedTime = 0;

  // for how small move the touch is recognized as click
  this.clickRange = 10;
  // minimum time for long click (in ms)
  this.longClickTime = 1000;
};

TouchMap.prototype.handleStart = function(evt) {
  logger.debug("touchstart.");
  evt.preventDefault();
  evt.stopPropagation();
  var self = this;
  var touches = evt.changedTouches;
  for (var index = 0; index < touches.length; index++) {
    var idx = self.ongoingTouchIndexById(touches[index].identifier);
    if (idx >= 0) {
      self.clearTouchData();
    }
  }

  for (var i = 0; i < touches.length; i++) {
    self.ongoingTouches.push(self.copyTouch(touches[i]));
    logger.debug("touchstart:" + i + ". " + touches[i].identifier);

    if (self.ongoingTouches.length === 1) {
      self.firstFingerId = touches[i].identifier;
      self.firstStartX = touches[i].clientX;
      self.firstStartY = touches[i].clientY;
      self.firstEndX = touches[i].clientX;
      self.firstEndY = touches[i].clientY;
      self.startCoord = self.map.getCenter();
      self.lastMoveDx = 0;
      self.lastMoveDy = 0;
      self.rightMenuOn = GuiConnector.isRightMenuVisible();
      GuiConnector.updateMouseCoordinates(touches[i].clientX,
          touches[i].clientY);
    }
    if (self.ongoingTouches.length === 2) {
      self.secondFingerId = touches[i].identifier;
      self.secondStartX = touches[i].clientX;
      self.secondStartY = touches[i].clientY;
      self.secondEndX = touches[i].clientX;
      self.secondEndY = touches[i].clientY;
      self.startZoom = self.map.getZoom();
      self.lastZoom = self.startZoom;

      for (var j = 0; j < self.ongoingTouches.length; j++) {
        if (self.ongoingTouches[j].identifier === self.firstFingerId) {
          self.firstStartX = self.ongoingTouches[j].clientX;
          self.firstStartY = self.ongoingTouches[j].clientY;
          self.startCoord = self.map.getCenter();
          self.lastMoveDx = 0;
          self.lastMoveDy = 0;
        }
      }
    }
    self.lastStartedFinger = touches[i].identifier;
    self.lastStartedTime = (new Date()).getTime();
  }

  // log3();

};

TouchMap.prototype.updateCoordinates = function(touch) {
  var self = this;
  if (touch.identifier === self.firstFingerId) {
    self.firstEndX = touch.clientX;
    self.firstEndY = touch.clientY;
    GuiConnector.updateMouseCoordinates(touch.clientX, touch.clientY);
  } else if (touch.identifier === self.secondFingerId) {
    self.secondEndX = touch.clientX;
    self.secondEndY = touch.clientY;
  }
};

TouchMap.prototype.lineDistance = function(x1, y1, x2, y2) {
  var xs = 0;
  var ys = 0;
  xs = x2 - x1;
  xs = xs * xs;
  ys = y2 - y1;
  ys = ys * ys;
  return Math.sqrt(xs + ys);
};

TouchMap.prototype.moveMap = function(dx, dy) {
  var self = this;
  self.map.panBy(dx - self.lastMoveDx, dy - self.lastMoveDy);
  self.lastMoveDx = dx;
  self.lastMoveDy = dy;
};

TouchMap.prototype.zoomMap = function(pointX, pointY, zoomLevel) {
  var self = this;
  if (self.lastZoom !== zoomLevel) {
    logger.debug(this.map.getDiv());
    logger.debug(this.map.getDiv().id);
    var id = PrimeFaces.escapeClientId(this.map.getDiv().id);
    logger.debug(id);
    self.lastZoom = zoomLevel;
    logger.debug("zoom6: " + zoomLevel);
    logger.debug("zoom6: " + $(id));
    var topPos = $(id).offset().top;
    var leftPos = $(id).offset().left;
    logger.debug("rel: " + leftPos + " - " + topPos);
    pointX -= leftPos;
    pointY -= topPos;
    var height = $(id).height();
    var width = $(id).width();
    logger.debug(pointX + " - " + pointY);
    logger.debug(width + " - " + height);
    var dx = width / 2 - pointX;
    var dy = height / 2 - pointY;
    self.map.panBy(-dx, -dy);
    self.map.setZoom(zoomLevel);
    self.map.panBy(dx, dy);
    logger.debug(dx + " - " + dy + " - " + zoomLevel);
  }
};

TouchMap.prototype.makeMove = function() {
  var self = this;
  if (self.firstFingerId !== null && self.firstFingerId !== undefined) {
    if (self.secondFingerId !== null && self.secondFingerId !== undefined) {
      var dist1 = self.lineDistance(self.secondStartX, self.secondStartY,
          self.firstStartX, self.firstStartY);
      var dist2 = self.lineDistance(self.secondEndX, self.secondEndY,
          self.firstEndX, self.firstEndY);
      var zoomFactor = dist2 / dist1;
      var changeLevels = Math.round((Math.log(zoomFactor) / Math.log(2)));
      self.zoomMap(self.firstEndX, self.firstEndY, changeLevels
          + self.startZoom);
    } else {
      var dx = -self.firstEndX + self.firstStartX;
      var dy = -self.firstEndY + self.firstStartY;
      var dist = Math.abs(dx) + Math.abs(dy);
      if (dist > self.clickRange) {
        self.moveMap(dx, dy);
      }
    }
  }
};

TouchMap.prototype.makeLeftClick = function(x, y) {
  logger.debug("Make left click on " + x + ", " + y + ".");
  var self = this;
  var el = $(document.elementFromPoint(x, y));
  logger.debug(el);

  // if we clicked on one of the elements on the map then emulate the click
  if (el.attr('src') !== undefined || el.attr('id') !== undefined
      || el.attr('title') !== undefined) {
    el.click();
  } else {
    var mev = {
      stop : null,
      latLng : self.getCustomMap().getMouseLatLng()
    };
    google.maps.event.trigger(self.map, 'click', mev);
  }

};

TouchMap.prototype.makeRightClick = function(x, y) {
  logger.debug("Make right click on " + x + ", " + y);
  var self = this;
  var el = $(document.elementFromPoint(x, y));

  // if we clicked on one of the elements on the map then emulate the click
  if (el.attr('src') !== undefined || el.attr('id') !== undefined
      || el.attr('title') !== undefined) {
    el.click();
  } else {
    var mev = {
      stop : null,
      latLng : self.getCustomMap().getMouseLatLng()
    };
    google.maps.event.trigger(self.map, 'rightclick', mev);
  }
};

TouchMap.prototype.handleEnd = function(evt) {
  evt.preventDefault();
  evt.stopPropagation();
  var touches = evt.changedTouches;
  var self = this;
  for (var i = 0; i < touches.length; i++) {
    var idx = self.ongoingTouchIndexById(touches[i].identifier);
    logger.debug("touch end:" + idx + "...");
    logger.debug("first finger: " + self.firstFingerId);
    logger.debug("last started: " + self.lastStartedFinger);

    var dist = Math.abs(self.firstEndX - self.firstStartX)
        + Math.abs(self.firstEndY - self.firstStartY);
    if (idx === self.firstFingerId && idx === self.lastStartedFinger
        && (dist < self.clickRange)) {
      var clickTime = (new Date().getTime() - self.lastStartedTime);
      logger.debug(clickTime + ", " + self.longClickTime);
      if (clickTime < self.longClickTime) {
        self.makeLeftClick(GuiConnector.xPos, GuiConnector.yPos);
      } else {
        self.makeRightClick(GuiConnector.xPos, GuiConnector.yPos);
      }
    }
    if (touches[i].identifier === self.firstFingerId) {
      self.firstFingerId = null;
    }
    if (touches[i].identifier === self.secondFingerId) {
      self.secondFingerId = null;
      self.startZoom = null;
      self.lastZoom = null;
      for (var j = 0; j < self.ongoingTouches.length; j++) {
        if (self.ongoingTouches[j].identifier === self.firstFingerId) {
          self.firstStartX = self.ongoingTouches[j].clientX;
          self.firstStartY = self.ongoingTouches[j].clientY;
          self.startCoord = self.map.getCenter();
        }
      }
    }
    if (idx >= 0) {
      self.ongoingTouches.splice(idx, 1); // remove it; we're done
    } else {
      logger.warn("can't figure out which touch to end: "
          + touches[i].identifier);
    }
  }
};

TouchMap.prototype.handleMove = function(evt) {
  evt.preventDefault();
  evt.stopPropagation();
  var touches = evt.changedTouches;
  var self = this;
  for (var i = 0; i < touches.length; i++) {
    var idx = self.ongoingTouchIndexById(touches[i].identifier);
    self.updateCoordinates(touches[i]);
    if (idx >= 0) {
      logger.debug("continuing touch " + idx + " " + touches[i].identifier);
      self.ongoingTouches.splice(idx, 1, self.copyTouch(touches[i])); // swap in
      // the new
      // touch
      // record

    } else {
      logger.warn("can't figure out which touch to continue"
          + touches[i].identifier);
    }
  }

  self.makeMove();

  if (self.rightMenuOn) {
    self.rightMenuOn = false;
    GuiConnector.hideRightClickMenu();
  }

  // clear logs
  logger.debug("", true);
  for (i = 0; i < self.ongoingTouches.length; i++) {
    var touch = self.ongoingTouches[i];
    logger.debug(touch.identifier + ": " + touch.clientX + "," + touch.clientY);
  }
};

TouchMap.prototype.handleCancel = function(evt) {
  var self = this;
  evt.preventDefault();
  evt.stopPropagation();
  var touches = evt.changedTouches;

  for (var i = 0; i < touches.length; i++) {
    self.ongoingTouches.splice(i, 1); // remove it; we're done
  }
};

TouchMap.prototype.copyTouch = function(touch) {
  return {
    identifier : touch.identifier,
    pageX : touch.pageX,
    pageY : touch.pageY,
    clientX : touch.clientX,
    clientY : touch.clientY
  };
};

TouchMap.prototype.ongoingTouchIndexById = function(idToFind) {
  var self = this;
  for (var i = 0; i < self.ongoingTouches.length; i++) {
    var id = self.ongoingTouches[i].identifier;

    if (id === idToFind) {
      // log(id+","+idToFind+","+i);
      return i;
    }
  }
  return -1; // not found
};

module.exports = TouchMap;
