"use strict";

var Helper = require('../Helper');

require("../mocha-config.js");

var AliasMarker = require('../../../main/js/map/marker/AliasMarker');
var AliasOverlay = require('../../../main/js/map/overlay/AliasOverlay');
var CustomMap = require('../../../main/js/map/CustomMap');
var IdentifiedElement = require('../../../main/js/map/data/IdentifiedElement');
var OverlayCollection = require('../../../main/js/map/overlay/OverlayCollection');
var ReactionOverlay = require('../../../main/js/map/overlay/ReactionOverlay');

var logger = require('./../logger');

var chai = require('chai');
var assert = chai.assert;

describe('CustomMap', function() {

  var helper;
  before(function() {
    helper = new Helper();
  });

  it("contructor", function() {
    var options = helper.createCustomMapOptions();
    var map = new CustomMap(options);
    assert.ok(map);
  });

  it("openLayoutById (not existing)", function() {
    var map = helper.createCustomMap();
    map.openLayoutById(-1);
    assert.equal(logger.getErrors().length, 1);
  });

  it("openLayout", function() {
    var options = helper.createCustomMapOptions();
    var model = options.getProject().getModel();
    var layout = model.getLayouts()[0];
    var map = new CustomMap(options);
    map.openLayoutById(layout.getId());
    assert.equal(logger.getErrors().length, 0);
  });

  it("addSelectedLayout", function() {
    var layoutId = 90;
    var retreiveActiveAliasesForLayoutCount = 0;
    var retreiveActiveReactionsForLayoutCount = 0;
    ServerConnector.addListener("onRetreiveActiveAliasesForLayout", function(e) {
      assert.equal(layoutId, e.param);
      retreiveActiveAliasesForLayoutCount++;
    });
    ServerConnector.addListener("onRetreiveActiveReactionsForLayout", function(e) {
      assert.equal(layoutId, e.param);
      retreiveActiveReactionsForLayoutCount++;
    });

    var map = helper.createCustomMap();
    map.addSelectedLayout(layoutId, "test layout");
    assert.equal(1, retreiveActiveAliasesForLayoutCount);
    assert.equal(1, retreiveActiveReactionsForLayoutCount);
    var vLayouts = JSON.parse(ServerConnector.getVisibleLayouts());
    assert.equal(1, vLayouts.length);
    assert.equal(layoutId, vLayouts[0]);
  });

  it("addAliasesForLayout", function() {
    var retreiveLightAliasesCounter = 0;
    ServerConnector.addListener("onRetreiveLightAliases", function(e) {
      assert.equal(2, e.param.length);
      assert.equal(18549, e.param[0][1]);
      assert.equal(18552, e.param[1][1]);
      retreiveLightAliasesCounter++;
    });
    var map = helper.createCustomMap();
    var jsonString = '[{"value":0.5,"color":{"value":-8388737,"falpha":0.0},"modelId":' + map.getId()
        + ',"idObject":18549},' + '{"value":-0.5,"color":{"value":-32897,"falpha":0.0},"modelId":' + map.getId()
        + ',"idObject":18552}]';
    map.addAliasesForLayout(1, jsonString);
    assert.ok(map.getModel().getLayoutDataById(1));
    assert.equal(2, map.getModel().getMissingAliasIds().length);

  });

  it("_showSelectedLayout", function() {
    var layoutId = 102;
    var map = helper.createCustomMap();

    // create layout
    map.getModel().initLayoutData(layoutId, "test layout");

    // add layout alias data
    var aliasObj = {
      idObject : 54,
      value : 0.4,
      color : 255,
      alpha : 0.0,
    };
    map.getModel().addAliasForLayout(layoutId, aliasObj);

    // add general alias data
    var aliasObj4 = {
      idObject : 54,
      modelId : map.getId(),
      bounds : {
        x : 10.0,
        y : 20.0,
        width : 30.0,
        height : 40.0,
      },
    };
    map.getModel().addAlias(aliasObj4);

    // add general reaction data
    var aliasObj3 = {
      idObject : 123,
      color : {
        value : 12423,
      },
      lines : [],
    };
    map.getModel().addReactionForLayout(layoutId, aliasObj3);

    // add reaction layout information
    var aliasObj2 = {
      idObject : 123,
      lines : [],
    };

    map.getModel().addReaction(aliasObj2);

    map.addSelectedLayout(layoutId, "test layout");

    map._showSelectedLayout(layoutId, 0, 1);

    assert.equal(2, map.selectedLayoutOverlays[layoutId].length);
    assert.ok(map.selectedLayoutOverlays[layoutId][0] instanceof AliasOverlay);
    assert.ok(map.selectedLayoutOverlays[layoutId][1] instanceof ReactionOverlay);

    // now hide the layout
    map._hideSelectedLayout(layoutId);
    assert.ok(map.selectedLayoutOverlays[layoutId]);
    assert.equal(0, map.selectedLayoutOverlays[layoutId].length);

  });
  it("openInfoWindowForAlias", function() {
    var layoutId = 102;
    var aliasId = 54;
    var retreiveFullAliasesCounter = 0;
    ServerConnector.addListener("onRetreiveFullAliases", function(e) {
      assert.equal(1, e.param.length);
      assert.equal(aliasId, e.param[0][1]);
      retreiveFullAliasesCounter++;
    });

    var map = helper.createCustomMap();

    // create layout
    map.getModel().initLayoutData(layoutId, "test layout");

    // add layout alias data
    var aliasObj = {
      idObject : aliasId,
      value : 0.4,
      color : 255,
      alpha : 0.0,
    };
    map.getModel().addAliasForLayout(layoutId, aliasObj);

    // add general alias data
    var aliasObj2 = {
      idObject : aliasId,
      bounds : {
        x : 10.0,
        y : 20.0,
        width : 30.0,
        height : 40.0
      },
      modelId : map.getId(),
    };
    map.getModel().addAlias(aliasObj2);

    map.openInfoWindowForAlias(aliasId, map.getId());
    assert.equal(1, retreiveFullAliasesCounter);
    assert.ok(map.getAliasInfoWindowById(aliasId));
    assert.ok(map.getAliasInfoWindowById(aliasId).isOpened());
  });

  it("openInfoWindowForMarker", function() {
    var aliasId = 54;
    var ovName = " overlay name";

    var onSendOverlayDetailDataRequest = 0;
    ServerConnector.addListener("onSendOverlayDetailDataRequest", function() {
      onSendOverlayDetailDataRequest++;
    });

    var map = helper.createCustomMap();

    // add general alias data
    var aliasObj = {
      idObject : aliasId,
      modelId : map.getId(),
      name : 'test',
      bounds : {
        x : 10.0,
        y : 20.0,
        width : 30.0,
        height : 40.0,
      },
    };
    map.getModel().addAlias(aliasObj);

    var oc = new OverlayCollection(map, ovName, true, false);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));
    oc.setResultsIds([ 'result_id' ]);

    map.renderOverlayCollection(oc);

    var aliasMarker = new AliasMarker(aliasId, "marker/empty.png", map.getModel().getAliasById(aliasId), map);

    map.openInfoWindowForMarker(aliasMarker);

    assert.equal(1, onSendOverlayDetailDataRequest);

    assert.ok(map.getAliasInfoWindowById(aliasId));
    assert.ok(map.getAliasInfoWindowById(aliasId).isOpened());

  });

  it("updateOverviewElementDetailData in OverlayCollection", function() {
    var aliasId = 54;
    var ovName = " overlay name";
    var searchResultId = 'result_id';

    var map = helper.createCustomMap();

    // add general alias data
    var aliasObj = {
      idObject : aliasId,
      modelId : map.getId(),
      name : 'test',
      bounds : {
        x : 10.0,
        y : 20.0,
        width : 30.0,
        height : 40.0,
      },
    };
    map.getModel().addAlias(aliasObj);

    var oc = new OverlayCollection(map, ovName, true, false);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));
    oc.setResultsIds([ searchResultId ]);

    map.renderOverlayCollection(oc);

    var aliasMarker = new AliasMarker(aliasId, "marker/empty.png", map.getModel().getAliasById(aliasId), map);
    map.getModel().getAliasById(aliasId).setType("RNA");

    map.openInfoWindowForMarker(aliasMarker);

    assert.ok(map.getAliasInfoWindowById(aliasId));
    assert.ok(map.getAliasInfoWindowById(aliasId).isOpened());

    var obj = {
      objectId : aliasId,
      modelId : map.getId(),
      type : "ALIAS"
    };

    var responseValue = "responseVVV";
    oc.updateOverviewElementDetailData(obj, searchResultId, {
      field : responseValue
    });

    assert.ok(map.getAliasInfoWindowById(aliasId).getContent().innerHTML.indexOf(responseValue) >= 0);
    assert.ok(map.getAliasInfoWindowById(aliasId).getContent().innerHTML.indexOf(searchResultId) >= 0);

  });

  it("retrieveMissingReactions", function() {
    var retreiveLightReactions = 0;
    var reactionId = 123;
    ServerConnector.addListener("onRetreiveLightReactions", function(e) {
      assert.equal(1, e.param.length);
      assert.equal(reactionId, e.param[0][1]);
      retreiveLightReactions++;
    });
    var layoutId = 102;
    var map = helper.createCustomMap();

    // create layout
    map.getModel().initLayoutData(layoutId, "test layout");

    // add general reaction data
    var aliasObj = {
      idObject : reactionId,
      color : {
        value : 12423,
      },
      lines : [],
    };
    map.getModel().addReactionForLayout(layoutId, aliasObj);

    map.retrieveMissingReactions();

    assert.equal(1, retreiveLightReactions);
  });

  it("registerSource 1", function() {
    var tmp = null;
    var map = helper.createCustomMap();

    map.registerSource = function(overlayCollection) {
      tmp = overlayCollection;
    };
    var oc = new OverlayCollection(map, 'testOV');
    assert.equal(tmp, oc);
  });

  it("registerSource 2", function() {
    var map = helper.createCustomMap();

    var overlays1 = 0;
    for ( var key1 in map.overlayCollections) {
      if (map.overlayCollections.hasOwnProperty(key1) && map.overlayCollections[key1] !== null
          && map.overlayCollections[key1] !== undefined) {
        overlays1++;
      }
    }
    assert.equal(0, overlays1);
    new OverlayCollection(map, 'testOV');

    var overlays2 = 0;
    for ( var key2 in map.overlayCollections) {
      if (map.overlayCollections.hasOwnProperty(key2) && map.overlayCollections[key2] !== null
          && map.overlayCollections[key2] !== undefined) {
        overlays2++;
      }
    }
    assert.equal(1, overlays2);

  });

  it("renderOverlayCollection", function() {
    var onRetreiveLightAliases = 0;
    ServerConnector.addListener("onRetreiveLightAliases", function() {
      onRetreiveLightAliases++;
    });

    var map = helper.createCustomMap();

    var oc = new OverlayCollection(map, 'name');

    oc.elements.push(new IdentifiedElement({
      objectId : 12,
      icon : "empty.png",
      modelId : map.getId(),
      type : "Alias"
    }));

    oc.elements.push(new IdentifiedElement({
      objectId : 13,
      icon : "empty.png",
      modelId : map.getId(),
      type : "Reaction"
    }));

    map.renderOverlayCollection(oc);

    assert.equal(1, onRetreiveLightAliases);
    assert.equal(12, map.getModel().getMissingAliasIds()[0]);

  });

  it("renderOverlayCollection for point", function() {
    var map = helper.createCustomMap();

    var oc = new OverlayCollection(map, 'name');

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));

    map.renderOverlayCollection(oc);

    assert.equal(false, ServerConnector.isWaitingForData());

    var markerCount = 0;
    for ( var id in oc.pointMarkers) {
      if (oc.pointMarkers.hasOwnProperty(id)) {
        markerCount++;
      }
    }

    assert.equal(1, markerCount);

  });

  it("renderOverlayCollection for reaction", function() {
    var map = helper.createCustomMap();

    var oc = new OverlayCollection(map, 'name');

    var javaObj = {
      objectId : "14",
      modelId : map.getId(),
      type : "Reaction",
    };

    oc.elements.push(new IdentifiedElement(javaObj));

    map.renderOverlayCollection(oc);

    assert.equal(false, ServerConnector.isWaitingForData());

    var markerCount = 0;
    for ( var id in oc.reactionMarkers) {
      if (oc.reactionMarkers.hasOwnProperty(id)) {
        markerCount++;
      }
    }

    assert.equal(1, markerCount);

  });

  it("clearData", function() {
    var map = helper.createCustomMap();

    var ovName = 'sopme name';

    var oc = new OverlayCollection(map, ovName);

    var javaObj = {
      objectId : "Point2D.Double[117.685546875, 204.6923828125001]",
      modelId : map.getId(),
      type : "POINT",
      icon : "marker/empty.png"
    };

    oc.elements.push(new IdentifiedElement(javaObj));

    map.renderOverlayCollection(oc);

    var onSendClearRequest = 0;
    ServerConnector.addListener("onSendClearRequest", function(e) {
      assert.equal(ovName, e.param);
      onSendClearRequest++;
    });

    map.clearData();

    var markerCount = 0;
    for ( var id in oc.pointMarkers) {
      if (oc.pointMarkers.hasOwnProperty(id)) {
        markerCount++;
      }
    }

    assert.equal(0, markerCount);
    assert.equal(1, onSendClearRequest);

  });

  it("getInfoWindowForIdentifiedElement ( reaction)", function() {
    var reactionId = 8;
    var map = helper.createCustomMap();

    var javaObj = {
      objectId : reactionId,
      modelId : map.getId(),
      type : "REACTION"
    };

    var ie = new IdentifiedElement(javaObj);
    var infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.equal(null, infoWindow);
    map._openInfoWindowForReaction(reactionId);

    infoWindow = map.getInfoWindowForIdentifiedElement(ie);

    assert.ok(infoWindow);
  });

  it("right click on map", function() {
    var map = helper.createCustomMap();

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'rightclick', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });
  it("left click on map", function() {
    var map = helper.createCustomMap();

    var mev = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(map.getActiveSubmapId());
    google.maps.event.trigger(map.getGoogleMap(), 'click', mev);
    assert.equal(map.getId(), map.getActiveSubmapId());

  });

  it("create polygon through drawingManager", function() {
    var map = helper.createCustomMap();

    map.turnOnDrawing();

    var triangleCoords = [ new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
        new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190), ];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths : triangleCoords,
      strokeColor : '#FF0000',
      strokeOpacity : 0.8,
      strokeWeight : 2,
      fillColor : '#FF0000',
      fillOpacity : 0.35
    });
    bermudaTriangle.setMap(map.getGoogleMap());

    var eventParam = {
      type : google.maps.drawing.OverlayType.POLYGON,
      overlay : bermudaTriangle,
    };

    // check creation complete behaviour
    assert.ok(map.isDrawingOn());
    assert.equal(google.maps.drawing.OverlayType.POLYGON, map._drawingManager.getDrawingMode());
    google.maps.event.trigger(map._drawingManager, 'overlaycomplete', eventParam);
    assert.ok(map.isDrawingOn());
    assert.equal(null, map._drawingManager.getDrawingMode());

    // and now check how it behaves when we wiught click on the polygon
    var clickEventParam = {
      stop : null,
      latLng : new google.maps.LatLng(40.0, -90.0)
    };

    assert.notOk(ServerConnector.getSelectedPolygon());
    assert.notOk(ServerConnector.getExportModelId());

    assert.notOk(map.getActiveSubmapId());
    assert.notOk(map.getSelectedArea());
    google.maps.event.trigger(bermudaTriangle, 'rightclick', clickEventParam);
    assert.ok(map.getSelectedArea());
    assert.equal(map.getId(), map.getActiveSubmapId());

    assert.ok(ServerConnector.getSelectedPolygon());
    assert.ok(ServerConnector.getExportModelId());

  });
});
