"use strict";

var Helper = require('../../Helper');

var LayoutData = require('../../../../main/js/map/data/LayoutData');
var LayoutAlias = require('../../../../main/js/map/data/LayoutAlias');
var MapModel = require('../../../../main/js/map/data/MapModel');
var Reaction = require('../../../../main/js/map/data/Reaction');
var assert = require('assert');

var logger = require('../../logger');

describe('MapModel', function() {
  var helper;
  before(function() {
    helper = new Helper();
  });

  it("MapModel constructor", function() {
    var model = new MapModel({
      idObject : 123
    });

    assert.equal(123, model.id);

    assert.ok(model.getAliases());
    assert.ok(model.getMissingAliasIds());
    assert.ok(model.getLayoutsData());
  });

  it("Adding layout alias", function() {
    var model = helper.createModel();

    var aliasObj = {
      idObject : 54,
      value : 0.4,
      color : 255,
      alpha : 0.0,
    };

    var layoutsCounter = Object.keys(model.getLayoutsData()).length;
    var missingAliasesCounter = Object.keys(model.getMissingAliasIds()).length;

    model.initLayoutData(123, 'test layout');
    model.addAliasForLayout(123, aliasObj);

    assert.equal(Object.keys(model.getLayoutsData()).length, layoutsCounter + 1);
    assert.equal(Object.keys(model.getMissingAliasIds()).length, missingAliasesCounter + 1);
  });

  it("Adding alias that was missing", function() {
    var model = helper.createModel();

    var aliasObj = {
      idObject : 123,
      value : 0.4,
      color : 255,
      alpha : 0.0,
    };

    assert.equal(0, model.getMissingAliasIds().length);

    model.initLayoutData(123, 'test layout');
    model.addAliasForLayout(123, aliasObj);

    var aliasObj2 = {
      idObject : 123,
      modelId : model.id,
      bounds : {
        x : 10.0,
        y : 20.0,
        width : 30.0,
        height : 40.0,
      },
    };
    assert.equal(1, model.getMissingAliasIds().length);
    model.addAlias(aliasObj2);
    assert.equal(0, model.getMissingAliasIds().length);
  });

  it("Adding reaction that was missing", function() {
    var model = helper.createModel();

    var aliasObj = {
      idObject : 123,
      modelId : model.id,
      value : 0.4,
      color : 255,
      alpha : 0.0,
    };

    assert.equal(0, model.getMissingReactionIds().length);

    model.initLayoutData(123, 'test layout');
    model.addReactionForLayout(123, aliasObj);

    var aliasObj2 = {
      idObject : 123,
      lines : [],
    };

    assert.equal(1, model.getMissingReactionIds().length);
    model.addReaction(aliasObj2);
    assert.equal(0, model.getMissingReactionIds().length);
  });

  it("Get layout by id", function() {
    var model = helper.createModel();

    var layoutData = model.getLayoutDataById(12);
    assert.equal(null, layoutData);

    model.initLayoutData(12, "test");

    layoutData = model.getLayoutDataById(12);
    assert.ok(layoutData);
    assert.ok(layoutData instanceof LayoutData);
  });

  it("init invalid layout data", function() {
    var model = helper.createModel();

    model.initLayoutData(12, "test");
    assert.equal(logger.getWarnings().length, 1);

    model.initLayoutData(12, "test");
    assert.equal(logger.getWarnings().length, 3);
  });

  it("addMissingAliasId", function() {
    var model = helper.createModel();

    model.addMissingAliasId(31);

    assert.equal(1, model.getMissingAliasIds().length);
    assert.equal(31, model.getMissingAliasIds()[0]);

    model.addMissingAliasId(31);

    assert.equal(1, model.getMissingAliasIds().length);
    assert.equal(31, model.getMissingAliasIds()[0]);

    var aliasObj = {
      idObject : 32,
      modelId : model.id,
      bounds : {
        x : 10.0,
        y : 20.0,
        width : 30.0,
        height : 40.0,
      },
    };

    assert.equal(1, model.getMissingAliasIds().length);
    model.addAlias(aliasObj);
    assert.equal(1, model.getMissingAliasIds().length);

    model.addMissingAliasId(32);

    assert.equal(1, model.getMissingAliasIds().length);

  });

  it("update alias", function() {
    var model = helper.createModel();

    var aliasObj = {
      idObject : 32,
      modelId : model.id,
    };

    model.addAlias(aliasObj);

    assert.equal(model.getAliasById(32).getName(), undefined);

    var aliasObj2 = {
      idObject : 32,
      modelId : model.id,
      name : "testName",
    };

    model.addAlias(aliasObj2);
    assert.equal(model.getAliasById(32).getName(), "testName");

  });

  it("addMissingReactionId", function() {
    var model = helper.createModel();

    model.addMissingReactionId(33);

    assert.equal(1, model.getMissingReactionIds().length);
    assert.equal(33, model.getMissingReactionIds()[0]);

    model.addMissingReactionId(33);

    assert.equal(1, model.getMissingReactionIds().length);
    assert.equal(33, model.getMissingReactionIds()[0]);

    var aliasObj = {
      idObject : 34,
      lines : [],
    };

    assert.equal(1, model.getMissingReactionIds().length);
    model.addReaction(aliasObj);
    assert.equal(1, model.getMissingReactionIds().length);

    model.addMissingReactionId(34);

    assert.equal(1, model.getMissingReactionIds().length);

  });

  it("getPointDataByPoint", function() {
    var model = helper.createModel();

    var point = new google.maps.Point(2, 3.45);
    var point2 = new google.maps.Point(2, 3.45);

    var data = model.getPointDataByPoint(point);

    var data2 = model.getPointDataByPoint(point2);

    assert.ok(data);
    assert.ok(data2);
    assert.equal(data, data2);
  });

  it("getPointDataBy invalid point", function() {
    var model = helper.createModel();

    var data = model.getPointDataByPoint({});

    assert.equal(data, null);
    assert.equal(logger.getWarnings().length, 1);
  });

  it("addReaction(Reaction)", function() {
    var reactionId = 125;
    var model = helper.createModel();

    var aliasObj = {
      idObject : reactionId,
      lines : [],
    };

    var reaction = new Reaction(aliasObj);
    assert.equal(null, model.getReactionById(reactionId));
    model.addReaction(reaction);
    assert.equal(reaction, model.getReactionById(reactionId));

  });

  it("addReaction(Reaction)", function() {
    var reactionId = 125;
    var model = helper.createModel();

    var aliasObj = {
      idObject : reactionId,
      lines : [],
    };

    var reaction = new Reaction(aliasObj);
    model.addReaction(reaction);
    assert.equal(logger.getWarnings().length, 0);
    model.addReaction(reaction);
    assert.equal(logger.getWarnings().length, 1);
  });

  it("updateAliasFor invalid Layout ", function() {
    var model = helper.createModel();

    var layoutId = 910;

    model.updateAliasForLayout(layoutId, {});

    assert.equal(logger.getErrors().length, 1);
  });

  it("updateAliasForLayout ", function() {
    var model = helper.createModel();

    var aliasId = 909;
    var javaObj = {
      idObject : aliasId,
      value : 0.4,
      color : {},
      geneVariations : [ {} ],
    };

    var layoutId = 910;

    model.initLayoutData(layoutId);

    model.getLayoutDataById(layoutId).addAlias(new LayoutAlias({
      idObject : aliasId
    }));

    model.updateAliasForLayout(layoutId, javaObj);

    assert.equal(logger.getErrors().length, 0);
    assert.equal(model.getLayoutDataById(layoutId).getAliasById(aliasId).getValue(), 0.4);
  });

});
